/******************************************************************************
Project:		LCMeter
File:			main.h
Author:			Philipp Burch
Date:			29.02.2008
Description:	Header file for main.c.
******************************************************************************/

#ifndef __MAIN_H__
#define __MAIN_H__

#include <stdint.h>

//Defines

//Flags for GlobalFlags
#define GF_TICK			0
#define GF_KEYPRESS		1
#define GF_GETFREQ		2
#define GF_GOTFREQ		3
#define GF_BUSY			4
#define GF_NOFREQ		5

#define L8(v)    (((uint16_t_u)(v)).a)
#define H8(v)    (((uint16_t_u)(v)).b)

#define L16(v)   (((uint32_t_s)(v)).a)
#define H16(v)   (((uint32_t_s)(v)).b)

#define LL8(v)   (((uint32_t_u)(v)).a)
#define LH8(v)   (((uint32_t_u)(v)).b)
#define HL8(v)   (((uint32_t_u)(v)).c)
#define HH8(v)   (((uint32_t_u)(v)).d)

//Pin definitions
#define RPO_CONTROL		PORTC
#define PIN_CTRL_CREF	PC0
#define PIN_CTRL_C		PC1
#define PIN_CTRL_LP		PC2
#define PIN_CTRL_LS		PC3
#define PIN_CTRL_EN		PC4
#define DDR_LX_CX		DDRC
#define RPO_LX_CX		PORTC
#define PIN_LX_CX		PC5

#define RPI_KEYS		PIND
#define PIN_KEY_MPLUS	PD0
#define PIN_KEY_MMINUS	PD1
#define PIN_KEY_PLUS	PD2
#define PIN_KEY_MINUS	PD3
#define PIN_KEY_START	PD4
#define PIN_KEY_CAL		PD6
#define PIN_KEY_MODE	PD7
#define RPI_CLOCK		PIND
#define PIN_CLOCK		PD5

//Globals
extern volatile uint8_t GlobalFlags;

typedef enum {
	CS_FIRSTCALL = 0,
	CS_MAINLOOP,
} call_src_t;

typedef struct {
	uint16_t delay;
	void (*callback)(call_src_t);
} event_t;

typedef enum {
	false = 0,
	true = 1
} bool;

typedef union {
  uint16_t word;
  struct {
    uint8_t a,b;
  };
} uint16_t_u;

typedef union {
  uint32_t dword;
  struct {
    uint8_t a,b,c,d;
  };
} uint32_t_u;

typedef union {
  uint32_t dword;
  struct {
    uint16_t a,b;
  };
} uint32_t_s;

//Prototypes

void set_event(uint16_t, void (*)(call_src_t));
void wait_ms(uint8_t);
void wait_20us();

#endif //__MAIN_H__
