/******************************************************************************
Project:		...
File:			lcd.c
Author:			Philipp Burch
Date:			16.12.2006
Description:	This file contains every function to access an attached LCD.
******************************************************************************/

#include <avr/io.h>
#include <stdint.h>
#include <string.h>
#include <avr/pgmspace.h>
#include <lcd.h>
#include <main.h>
#include <math.h>

char lcd_buf[17];
uint8_t lcd_cursor;

void lcd_init()
{
	uint8_t i, j;

    #if LCD_PWRDELAY
		wait_ms(250);
    #endif
	for(j = 0; j < 2; ++j) {
		wait_ms(100);
        #if LCD_PWRDELAY
            //Power up wait
			wait_ms(150);
        #endif

		LCD_PCONTROL &= ~_BV(LCD_RS);
		for(i = 0; i < 3; ++i) {
			LCD_PDATA &= ~LCD_NIBBLE;
			#if LCD_NIBBLE == 0x0F
				LCD_PDATA |= 0x03;
			#else
				LCD_PDATA |= 0x30;
			#endif
			lcd_enable();
			wait_ms(10);
		}
		LCD_PDATA &= ~LCD_NIBBLE;
		#if LCD_NIBBLE == 0x0F
			LCD_PDATA |= 0x02;
		#else
			LCD_PDATA |= 0x20;
		#endif
		lcd_enable();
		wait_ms(10);

		for (i = 0; i < 3; ++i) {
			#if LCD_2X8
				lcd_cmd(0x28);
			#else
				lcd_cmd(0x20);
			#endif
			wait_ms(5);

			lcd_cmd(0x01);
			wait_ms(5);

			lcd_cmd(0x06);
			wait_ms(5);

			lcd_cmd(0x0d);
			wait_ms(10);
		}
	}
}

char *lcd_buffer() {
	return lcd_buf;
}

void lcd_settext_P(const prog_char *text) {
	/*strcpy_P(lcd_buf, text);
	uint8_t i = strlen_P(text);
	for (; i < 16; ++i) lcd_buf[i] = ' ';*/

	uint8_t i;
	bool f = false;
	for (i = 0; i < 16; ++i) {
		if (!f) {
			if (pgm_read_byte(&text[i])) {
				lcd_buf[i] = pgm_read_byte(&text[i]);
			} else {
				f = true;
				lcd_buf[i] = ' ';
			}
		} else {
			lcd_buf[i] = ' ';
		}
	}
}

void lcd_setcursor(uint8_t cursor) {
	if (cursor > 15) {
		cursor = 15;
		lcd_cmd(0x0c);
	} else {
		lcd_cmd(0x0d);
	}
	lcd_cursor = cursor;
}

void lcd_cmd(const uint8_t command)
{
	//RS low
	LCD_PCONTROL &= ~_BV(LCD_RS);

	lcd_putbyte(command);
}

uint8_t lcd_data(uint8_t data)
{
	//RS high
	LCD_PCONTROL |= _BV(LCD_RS);

	return lcd_putbyte(data);
}


uint8_t lcd_putbyte(uint8_t byte)
{
	//High nibble
	LCD_PDATA &= ~LCD_NIBBLE;
	#if LCD_NIBBLE == 0x0F
		LCD_PDATA |= byte >> 4;
	#else
		LCD_PDATA |= byte & 0xF0;
	#endif
	lcd_enable();
	
	//Low nibble
	LCD_PDATA &= ~LCD_NIBBLE;
	#if LCD_NIBBLE == 0x0F
		LCD_PDATA |= byte & 0x0F;
	#else
		LCD_PDATA |= byte << 4;
	#endif
	lcd_enable();

	return byte;
}

void lcd_enable()
{
	LCD_PCONTROL |= _BV(LCD_E);
	wait_20us();
	LCD_PCONTROL &= ~_BV(LCD_E);
	wait_20us();
}

void lcd_cursorhome()
{
	lcd_cmd(0x80);
}

void lcd_setddram(uint8_t ddaddr)
{
	lcd_cmd(0x80 | (ddaddr & 0x7F));
}

void lcd_update() {
	static uint8_t pos = 0;
#if LCD_2X8
	if (pos < 8) {
#endif
		lcd_setddram(pos);
#if LCD_2X8
	} else {
		lcd_setddram(pos - 8 + 0x40);
	}
#endif
	lcd_data(lcd_buf[pos]);
#if LCD_2X8
	if (lcd_cursor < 8) {
#endif
		lcd_setddram(lcd_cursor);
#if LCD_2X8
	} else {
		lcd_setddram(lcd_cursor - 8 + 0x40);
	}
#endif
	pos = (pos + 1) & 0x0f;
}

void lcd_printval(const prog_char *prefix, float value, const prog_char *suffix, uint8_t precision) {
	char *bptr = lcd_buf;
	static prog_char pref[] = "pnm kMGT";	// -> 
	uint8_t exp = 4, dp = 0;

	strcpy_P(bptr, prefix);
	bptr += strlen_P(prefix);

	if (value < 0) {
		*bptr++ = '-';
		value = fabs(value);
	}

	if (value != 0.0) {
		//Get exponent
		while (value < 1.0) {
			value *= 1000.0;
			--exp;
		}
		while (value >= 1000.0) {
			value /= 1000.0;
			++exp;
		}
	}

	if (exp > 8) {
		exp = 0;
		value = 0.0;
		bptr = lcd_buf + strlen_P(prefix);
	}

	//Normalize (x.xxx)
	while (value >= 10.0) {
		value /= 10.0;
		++dp;
	}

	uint8_t num;
	for (uint8_t i = 0; i < precision; ++i) {
		num = (uint8_t)value;
		*bptr++ = '0' + num;
		value = (value - num) * 10;
		if (!dp) {
			*bptr++ = '.';
		}
		--dp;
	}

	if (exp != 4) *bptr++ = pgm_read_byte(&pref[exp]);

	strcpy_P(bptr, suffix);
	bptr += strlen_P(suffix);
	for (; bptr < lcd_buf + 16; *bptr++ = ' ');
}






