/******************************************************************************
Project:		LCMeter
File:			lcd.c
Author:			Philipp Burch
Date:			29.02.2008
Description:	This is the main code file. Everything starts here.
******************************************************************************/

#include <main.h>
#include <lcd.h>
#include <lc.h>
#include <lcd.h>
#include <key.h>
#include <stdint.h>
#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>
#include <util/delay.h>

volatile uint8_t GlobalFlags = 0;
volatile uint8_t keys;
event_t Event;

int main() {
	wdt_enable(WDTO_2S);

	PORTB = 0x00;
	DDRB = 0x3f;

	PORTC = 0x1f;
	DDRC = 0x3f;

	PORTD = 0xdf;
	DDRD = 0x00;

	//Timebase
	TCCR2 = _BV(WGM21) | _BV(CS22) | _BV(CS21) | _BV(CS20);
	OCR2 = 24;		//625Hz at 16MHz
	TIMSK = _BV(OCIE2);

	lc_init();
	lcd_init();

	lcd_setcursor(16);		//Invisible
	lc_calibrate(CS_FIRSTCALL);

	uint8_t f;
	for(;;) {
		cli();
		f = GlobalFlags;
		GlobalFlags &= ~(_BV(GF_TICK) | _BV(GF_KEYPRESS) | _BV(GF_GOTFREQ));
		sei();

		wdt_reset();

		if (f & _BV(GF_TICK)) {
			lcd_update();

			if (f & _BV(GF_KEYPRESS) && !(f & _BV(GF_BUSY))) {
				key_process(keys);
			}

			if (Event.delay && !--Event.delay) Event.callback(CS_MAINLOOP);
		}

		if (f & _BV(GF_GOTFREQ)) lc_get_frequency(CS_MAINLOOP);
	}
}

void set_event(uint16_t delay, void (*callback)(call_src_t)) {
	if (Event.delay) return;
	Event.delay = delay;
	Event.callback = callback;
}

void wait_ms(uint8_t ms) {
	for (; ms; --ms) _delay_ms(1);
}

void wait_20us() {
	_delay_us(20);
}


ISR(TIMER2_COMP_vect) {
	static uint8_t freq_delay = 0;

	if (GlobalFlags & _BV(GF_GETFREQ)) {
		if (!freq_delay) {
			GlobalFlags &= ~_BV(GF_GOTFREQ);
			freq_delay = 126;	//200ms
			TCNT1 = 0;
			TCCR1B = _BV(CS12)  | _BV(CS11) | _BV(CS10);
		}
		if (!--freq_delay) {
			TCCR1B = 0;
			GlobalFlags &= ~_BV(GF_GETFREQ);
			GlobalFlags |= _BV(GF_GOTFREQ);
		}
	}

	GlobalFlags |= _BV(GF_TICK);
	if ((keys = key_update())) GlobalFlags |= _BV(GF_KEYPRESS);
}
